from .along_axis import get_executor_ls_by_block_along_axis as along_axis
from .along_diagonal import get_executor_ls_by_block_along_diagonal as along_diagonal
from .of_all import get_executor_ls_by_block_of_all as of_all
from .of_triangle import get_executor_ls_by_block_of_triangle as of_triangle
