metadata = {'sep': '\t', 'title': 'this is the title', 'column_name': ['epoch', 'loss', 'model_name'],
            'column_type': ['int', 'float', 'str'], 'column_num': 3}
content = {'epoch': [0, 1, 10, 11],
           'loss': [2.31, 2.22, 12.31, 12.22],
           'model_name': ['m0.pkt', 'm1.pkt', 'm01.pkt', 'm11.pkt']}
